"""Integral domains."""

from enum import Enum


class Domain(Enum):
    """A domain of an integral."""

    Interval = 0
    Triangle = 1
    Quadrilateral = 2
    Tetrahedron = 3
    Hexahedron = 4



"""Gauss--Legendre quadrature rule."""

import numpy as np
import numpy.typing as npt
from quadraturerules.domain import Domain
import typing


def gauss_legendre(
    domain: Domain,
    order: int,
) -> typing.Tuple[npt.NDArray[np.float64], npt.NDArray[np.float64]]:
    """Get a Gauss--Legendre quadrature rule."""
    match domain:
        case Domain.Interval:
            match order:
                case 1:
                    return np.array([[0.5, 0.5]]), np.array([1.0])  # noqa: E501
                case 2:
                    return np.array(
                        [
                            [0.21132486540518708, 0.7886751345948129],
                            [0.7886751345948129, 0.21132486540518708],
                        ]
                    ), np.array([0.5, 0.5])  # noqa: E501
                case 3:
                    return np.array(
                        [
                            [0.5, 0.5],
                            [0.11270166537925835, 0.8872983346207417],
                            [0.8872983346207417, 0.11270166537925835],
                        ]
                    ), np.array([0.4444444444444444, 0.2777777777777778, 0.2777777777777778])  # noqa: E501
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Triangle:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Quadrilateral:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Tetrahedron:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")
        case Domain.Hexahedron:
            match order:
                case _:
                    raise ValueError(f"Invalid order: {order}")



"""Quadrature rules."""

from enum import Enum as _Enum
import typing as _typing
import numpy.typing as _npt
import numpy as _np

from quadraturerules import rules
from quadraturerules.domain import Domain


class QuadratureRule(_Enum):
    """A quadrature rule family."""

    GaussLegendre = 1


def single_integral_quadrature(
    rtype: QuadratureRule,
    domain: Domain,
    order: int,
) -> _typing.Tuple[_npt.NDArray[_np.float64], _npt.NDArray[_np.float64]]:
    """Get a quadrature rule for a single integral."""
    match rtype:
        case QuadratureRule.GaussLegendre:
            return rules.gauss_legendre(domain, order)
        case _:
            raise ValueError(f"Unsupported rule for single integral: {rtype}")



import pytest
import numpy as np
from quadraturerules import Domain, QuadratureRule, single_integral_quadrature


@pytest.mark.parametrize("order", [1, 2, 3])
def test_sum_weights(order):
    assert np.isclose(
        sum(single_integral_quadrature(QuadratureRule.GaussLegendre, Domain.Interval, order)[1]),
        1.0,
    )
